from django.db.models import Count, F, Q, IntegerField
from django.db.models.functions import Cast

import sal.plugin

HEALTH_Q = Q(facts__management_source__name='Puppet', facts__fact_name='mac_macbook_batteryhealth')

class BatteryHealth(sal.plugin.Widget):
    description = 'Macbook battery health'
    widget_width = 4
    supported_os_families = [sal.plugin.OSFamilies.darwin]

    def get_context(self, machines, **kwargs):
        context = self.super_get_context(machines, **kwargs)
        health = (machines.filter(HEALTH_Q)
                  .values(health=F('facts__fact_data')))
        values = [min(100, int(x['health'])) for x in health]
        binsize = 5
        bins = {binsize * x: 0 for x in range(1 + 100 // binsize)}
        for v in values:
            for b in bins:
                if b >= v:
                    bins[b] += 1
                    break
        context['data'] = [{'label': f'{k}%', 'value': v} for k, v in bins.items()]
        return context

    def filter(self, machines, data):

        data = int(data.replace('%', ''))
        machines = (machines.filter(HEALTH_Q)
                    .annotate(health=Cast('facts__fact_data', IntegerField()))
                    .filter(health__lte=data))
        return machines, f'Machines with {data}% battery health or less'
